package com.tzld.piaoquan.wecom.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.wecom.dao.mapper.UserMapper;
import com.tzld.piaoquan.wecom.model.po.User;
import com.tzld.piaoquan.wecom.model.po.UserExample;
import com.tzld.piaoquan.wecom.service.AccessTokenService;
import com.tzld.piaoquan.wecom.utils.HttpClientUtil;
import com.tzld.piaoquan.wecom.utils.HttpPoolClient;
import com.tzld.piaoquan.wecom.utils.page.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@RestController
@RequestMapping("/test")
public class TestController {

    private static final HttpPoolClient httpPoolClientDefault = HttpClientUtil.create(30000, 30000, 2000, 5000, 5, 30000);

    @Autowired
    private AccessTokenService accessTokenService;

    @Autowired
    private UserMapper userMapper;


    @GetMapping("/staff")
    public void staff() throws IOException {
        String weComAccessToken = accessTokenService.getWeComAccessToken();
        log.info("weComAccessToken={}", weComAccessToken);
        if (weComAccessToken == null) {
            return;
        }
        String url = "https://qyapi.weixin.qq.com/cgi-bin/externalcontact/get_follow_user_list?access_token=" + weComAccessToken;
        String s = httpPoolClientDefault.get(url);
        log.info("staff={}", s);

    }

    @GetMapping("/user")
    public void user() throws IOException {
        String weComAccessToken = accessTokenService.getWeComAccessToken();
        log.info("weComAccessToken={}", weComAccessToken);
        if (weComAccessToken == null) {
            return;
        }
        String url = String.format("https://qyapi.weixin.qq.com/cgi-bin/externalcontact/list?access_token=%s&userid=%s", weComAccessToken, "SongYi");
        String s = httpPoolClientDefault.get(url);
        log.info("user={}", s);

    }

    @GetMapping("/userListDetail")
    public void userListDetail() throws IOException {
        String weComAccessToken = accessTokenService.getWeComAccessToken();
        log.info("weComAccessToken={}", weComAccessToken);
        if (weComAccessToken == null) {
            return;
        }
        String url = String.format("https://qyapi.weixin.qq.com/cgi-bin/externalcontact/batch/get_by_user?access_token=%s", weComAccessToken);
        JSONObject jsonObject = new JSONObject();
        JSONArray jsonArray = new JSONArray();
        jsonArray.add("SongYi");
        jsonObject.put("userid_list", jsonArray);
        jsonObject.put("limit", 10);
        String post = httpPoolClientDefault.post(url, jsonObject.toJSONString());
        log.info("userListDetail={}", post);
    }

    @GetMapping("/userDetail")
    public void userDetail(@RequestParam String userId) throws IOException {
        String weComAccessToken = accessTokenService.getWeComAccessToken();
        log.info("weComAccessToken={}", weComAccessToken);
        if (weComAccessToken == null) {
            return;
        }
        String url = String.format("https://qyapi.weixin.qq.com/cgi-bin/externalcontact/get?access_token=%s&external_userid=%s", weComAccessToken, userId);
        String get = httpPoolClientDefault.get(url);
        log.info("userDetail={}", get);
    }


    @GetMapping("/getUserId")
    public void getUserId() throws IOException {
        UserExample example = new UserExample();
        Page page = new Page<>(1, 10);
        example.setPage(page);
        List<User> userList = userMapper.selectByExample(example);
        if(CollectionUtils.isEmpty(userList)){
            return;
        }
        List<String> collect = userList.stream().map(User::getExternalUserId3rdParty).collect(Collectors.toList());
        String s = collect.get(0);
        String weComAccessToken = accessTokenService.getWeComAccessToken();
        log.info("weComAccessToken={}", weComAccessToken);
        if (weComAccessToken == null) {
            return;
        }
        String url = String.format("https://qyapi.weixin.qq.com/cgi-bin/externalcontact/from_service_external_userid?access_token=%s", weComAccessToken);
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("external_userid", s);
        jsonObject.put("source_agentid", 1000009);
        String post = httpPoolClientDefault.post(url, jsonObject.toJSONString());
        log.info("getUserId={}", post);
    }
}
